; Sketchy Example Program
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the Sketchy distribution
; for conditions of use.

; ---name---
; iota

; ---conformance---
; Sketchy Extension

; ---purpose---
; Generate a sequence of natural numbers.
; The resulting sequence starts with 1
; and is in ascending order.

; ---args---
; N - maximum number in sequence.

; ---keywords---
; IOTA function, sequence of numbers, arithmetics, natural

; ---see-also---
; digits, sum, product, sort, unsort

; ---example---
; (iota 10) => (1 2 3 4 5 6 7 8 9 10)

;c lib/digits.l
;c lib/list.l
:c lib/append.l
;c lib/natural.l
;c lib/zerop.l zero?

; ---model---
; (define (iota x)
;   (cond ((zero? x) '())
;     (#t (append (iota (n- x 1)) (list x)))))

; ---code---
(define (iota x)
  (letrec
    ((j (lambda (x r)
      (cond ((zero? x) r)
        (#t (j (n- x 1) (cons x r)))))))
    (j (natural x) '())))

